import { initStore } from 'src/node.polyfill';
import * as entities from 'src/lib/storage/entities';
import * as actions from 'src/actions';
import { JolocomLib } from 'jolocom-lib';
import { interactionHandlers } from 'src/lib/storage/interactionTokens';
import { withErrorHandler } from './actions/modifiers';
export { initStore, entities, actions };
const initErrorHandler = (error) => dispatch => {
    console.error(error.message);
    return Promise.reject(error);
};
export class JolocomSDK {
    constructor(store) {
        this.store = store;
        this.dispatch = this.store.dispatch;
        this.idw = this.store.backendMiddleware.identityWallet;
    }
    /**
     * Returns an agent with an Identity provided by a store
     *
     * @remarks
     * This depends on an existing database connection
     *
     * @param store - The agent state with storage connection
     * @returns An Agent with the identity existing in the storage in store
     */
    static async fromStore(store) {
        await store.dispatch(withErrorHandler(initErrorHandler)(actions.accountActions.checkIdentityExists));
        return new JolocomSDK(store);
    }
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    static async fromMnemonic(mnemonic) {
        const store = initStore();
        await store.dispatch(actions.registrationActions.recoverIdentity(mnemonic));
        return new JolocomSDK(store);
    }
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    static async newDIDFromSeed(seed) {
        const store = initStore();
        await store.dispatch(actions.registrationActions.createIdentity(seed.toString('hex')));
        return new JolocomSDK(store);
    }
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    async tokenRecieved(jwt) {
        const token = JolocomLib.parse.interactionToken.fromJWT(jwt);
        await this.store.backendMiddleware.storageLib.store.interactionToken(token);
        await this.dispatch(interactionHandlers[token.interactionType](token));
    }
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    async credRequestToken(request) {
        const token = await this.idw.create.interactionTokens.request.share(request, await this.store.backendMiddleware.keyChainLib.getPassword());
        this.store.backendMiddleware.storageLib.store.interactionToken(token);
        return token.encode();
    }
    /**
     * Returns a base64 encoded signed credential offer token, given
     * request attributes
     *
     * @param offer - credential offer attributes
     * @returns A base64 encoded signed credential offer token offering
     * credentials according to `offer`
     */
    async credOfferToken(offer) {
        const token = await this.idw.create.interactionTokens.request.offer(offer, await this.store.backendMiddleware.keyChainLib.getPassword());
        this.store.backendMiddleware.storageLib.store.interactionToken(token);
        return token.encode();
    }
    /**
     * Returns a base64 encoded signed credential issuance token, given
     * issuance attributes and a recieved token selecting desired issuance
     *
     * @param issuance - credential issuance attributes
     * @param selection - base64 encoded credential offer response token
     * @returns A base64 encoded signed issuance token containing verifiable
     * credentials
     */
    async credIssuanceToken(issuance, selection) {
        const token = await this.idw.create.interactionTokens.response.issue(issuance, await this.store.backendMiddleware.keyChainLib.getPassword(), await JolocomLib.parse.interactionToken.fromJWT(selection));
        this.store.backendMiddleware.storageLib.store.interactionToken(token);
        return token.encode();
    }
}
