import { createConnection, } from 'typeorm/browser';
import { plainToClass } from 'class-transformer';
import { entityList, SettingEntity, PersonaEntity, MasterKeyEntity, VerifiableCredentialEntity, SignatureEntity, CredentialEntity, CacheEntity, InteractionTokenEntity, } from 'src/lib/storage/entities';
import { DidDocument } from 'jolocom-lib/js/identity/didDocument/didDocument';
import { groupAttributesByCredentialId } from './utils';
import migrationsList from './migration';
import { JolocomLib } from 'jolocom-lib';
export class Storage {
    constructor(config) {
        this.store = {
            setting: this.saveSetting.bind(this),
            persona: this.storePersonaFromJSON.bind(this),
            verifiableCredential: this.storeVClaim.bind(this),
            encryptedSeed: this.storeEncryptedSeed.bind(this),
            credentialMetadata: (metadata) => this.createConnectionIfNeeded().then(() => storeCredentialMetadata(this.connection)(metadata)),
            issuerProfile: (issuer) => this.createConnectionIfNeeded().then(() => storeIssuerProfile(this.connection)(issuer)),
            didDoc: (doc) => this.createConnectionIfNeeded().then(() => cacheDIDDoc(this.connection)(doc)),
            interactionToken: (token) => this.createConnectionIfNeeded().then(() => storeInteractionToken(this.connection)(token)),
        };
        this.get = {
            settingsObject: this.getSettingsObject.bind(this),
            setting: this.getSetting.bind(this),
            persona: this.getPersonas.bind(this),
            verifiableCredential: this.getVCredential.bind(this),
            attributesByType: this.getAttributesByType.bind(this),
            vCredentialsByAttributeValue: this.getVCredentialsForAttribute.bind(this),
            encryptedSeed: this.getEncryptedSeed.bind(this),
            credentialMetadata: (credential) => this.createConnectionIfNeeded().then(() => getMetadataForCredential(this.connection)(credential)),
            publicProfile: (did) => this.createConnectionIfNeeded().then(() => getPublicProfile(this.connection)(did)),
            didDoc: (did) => this.createConnectionIfNeeded().then(() => getCachedDIDDoc(this.connection)(did)),
            interactionTokens: (nonce) => this.createConnectionIfNeeded().then(() => getTokensByNonce(this.connection)(nonce)),
        };
        this.delete = {
            verifiableCredential: this.deleteVCred.bind(this),
        };
        this.initConnection = this.createConnectionIfNeeded.bind(this);
        this.config = Object.assign({}, config, { entities: entityList, migrations: migrationsList });
    }
    async resetDatabase() {
        await this.createConnectionIfNeeded();
        await this.connection.dropDatabase();
        await this.connection.close();
        this.connectionPromise = null;
        await this.createConnectionIfNeeded();
    }
    async createConnectionIfNeeded() {
        if (this.connectionPromise)
            return this.connectionPromise;
        return (this.connectionPromise = createConnection(this.config)
            .then(conn => {
            this.connection = conn;
            return conn;
        })
            .catch(err => {
            this.connectionPromise = null;
            throw err;
        }));
    }
    async getSettingsObject() {
        await this.createConnectionIfNeeded();
        const settingsList = await this.connection.manager.find(SettingEntity);
        const settings = {};
        settingsList.forEach(setting => {
            settings[setting.key] = setting.value;
        });
        return settings;
    }
    async getSetting(key) {
        await this.createConnectionIfNeeded();
        const setting = await this.connection.manager.findOne(SettingEntity, {
            key,
        });
        if (setting)
            return setting.value;
    }
    async saveSetting(key, value) {
        await this.createConnectionIfNeeded();
        const repo = this.connection.getRepository(SettingEntity);
        const setting = repo.create({ key, value });
        await repo.save(setting);
    }
    // TODO: refactor needed on multiple personas
    async getPersonas(query) {
        await this.createConnectionIfNeeded();
        return this.connection.manager.find(PersonaEntity);
    }
    async getVCredential(query) {
        await this.createConnectionIfNeeded();
        const entities = await this.connection.manager.find(VerifiableCredentialEntity, {
            where: query,
            relations: ['claim', 'proof', 'subject'],
        });
        return entities.map(e => e.toVerifiableCredential());
    }
    async getAttributesByType(type) {
        await this.createConnectionIfNeeded();
        const localAttributes = await this.connection
            .getRepository(CredentialEntity)
            .createQueryBuilder('credential')
            .leftJoinAndSelect('credential.verifiableCredential', 'verifiableCredential')
            .where('verifiableCredential.type = :type', { type })
            .getMany();
        const results = groupAttributesByCredentialId(localAttributes).map(entry => ({
            verification: entry.verifiableCredential.id,
            values: entry.propertyValue,
            fieldName: entry.propertyName,
        }));
        return { type, results };
    }
    async getVCredentialsForAttribute(attribute) {
        await this.createConnectionIfNeeded();
        const entities = await this.connection
            .getRepository(VerifiableCredentialEntity)
            .createQueryBuilder('verifiableCredential')
            .leftJoinAndSelect('verifiableCredential.claim', 'claim')
            .leftJoinAndSelect('verifiableCredential.proof', 'proof')
            .leftJoinAndSelect('verifiableCredential.subject', 'subject')
            .where('claim.propertyValue = :attribute', { attribute })
            .getMany();
        return entities.map(e => e.toVerifiableCredential());
    }
    async getEncryptedSeed() {
        await this.createConnectionIfNeeded();
        const masterKeyEntity = await this.connection.manager.find(MasterKeyEntity);
        if (masterKeyEntity.length) {
            return masterKeyEntity[0].encryptedEntropy;
        }
        return null;
    }
    async storePersonaFromJSON(args) {
        await this.createConnectionIfNeeded();
        const persona = plainToClass(PersonaEntity, args);
        await this.connection.manager.save(persona);
    }
    async storeEncryptedSeed(args) {
        await this.createConnectionIfNeeded();
        const encryptedSeed = plainToClass(MasterKeyEntity, args);
        await this.connection.manager.save(encryptedSeed);
    }
    async storeVClaim(vCred) {
        await this.createConnectionIfNeeded();
        const verifiableCredential = VerifiableCredentialEntity.fromVerifiableCredential(vCred);
        const signature = SignatureEntity.fromLinkedDataSignature(vCred.proof);
        const claims = CredentialEntity.fromVerifiableCredential(vCred);
        claims.forEach(claim => (claim.verifiableCredential = verifiableCredential));
        signature.verifiableCredential = verifiableCredential;
        verifiableCredential.proof = [signature];
        verifiableCredential.claim = claims;
        await this.connection.manager.save(verifiableCredential);
    }
    async deleteVCred(id) {
        await this.createConnectionIfNeeded();
        await this.connection.manager
            .createQueryBuilder()
            .delete()
            .from(CredentialEntity)
            .where('verifiableCredential = :id', { id })
            .execute();
        await this.connection.manager
            .createQueryBuilder()
            .delete()
            .from(SignatureEntity)
            .where('verifiableCredential = :id', { id })
            .delete()
            .execute();
        await this.connection.manager
            .createQueryBuilder()
            .delete()
            .from(VerifiableCredentialEntity)
            .where('id = :id', { id })
            .execute();
    }
}
const cacheDIDDoc = (connection) => (doc) => {
    const cacheEntry = plainToClass(CacheEntity, {
        key: `didCache:${doc.did}`,
        value: doc.toJSON(),
    });
    return connection.manager.save(cacheEntry);
};
const getCachedDIDDoc = (connection) => async (did) => {
    const [entry] = await connection.manager.findByIds(CacheEntity, [
        `didCache:${did}`,
    ]);
    try {
        return DidDocument.fromJSON(entry.value);
    }
    catch (err) {
        return undefined;
    }
};
const storeInteractionToken = (connection) => async (token) => {
    const tokenEntry = InteractionTokenEntity.fromJWT(token);
    return connection.manager.save(tokenEntry);
};
const getTokensByNonce = (connection) => async (nonce) => 
// return await connection.manager.find(InteractionTokenEntity)
connection.manager
    .find(InteractionTokenEntity, {
    where: [{ nonce }],
})
    .then(entities => entities.map(entity => JolocomLib.parse.interactionToken.fromJWT(entity.original)));
const storeCredentialMetadata = (connection) => (credentialMetadata) => {
    const { issuer, type: credentialType } = credentialMetadata;
    const cacheEntry = plainToClass(CacheEntity, {
        key: buildMetadataKey(issuer.did, credentialType),
        value: Object.assign({}, credentialMetadata, { issuer: credentialMetadata.issuer.did }),
    });
    return connection.manager.save(cacheEntry);
};
const getMetadataForCredential = (connection) => async ({ issuer, type: credentialType, }) => {
    const entryKey = buildMetadataKey(issuer, credentialType);
    const [entry] = await connection.manager.findByIds(CacheEntity, [entryKey]);
    return (entry && entry.value) || {};
};
const buildMetadataKey = (issuer, credentialType) => {
    if (typeof credentialType === 'string') {
        return `${issuer}${credentialType}`;
    }
    return `${issuer}${credentialType[credentialType.length - 1]}`;
};
const storeIssuerProfile = (connection) => (issuer) => {
    const cacheEntry = plainToClass(CacheEntity, {
        key: issuer.did,
        value: issuer,
    });
    return connection.manager.save(cacheEntry);
};
const getPublicProfile = (connection) => async (did) => {
    const [issuerProfile] = await connection.manager.findByIds(CacheEntity, [did]);
    return (issuerProfile && issuerProfile.value) || { did };
};
